/************************************************************************/
/*                                                                      */
/*                      Reading rotary encoder                   	*/
/*                      one, two and four step encoders supported	*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                                                                      */
/************************************************************************/

//Edit zu Testzwecken durch screwdriver
//Not quick, but very dirty!

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include <avr/pgmspace.h>

#include "uart.h"

#define	XTAL		4e6			// 4MHz

/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 4000000UL
#endif

/* 9600 baud */
#define UART_BAUD_RATE      9600     

#define PHASE_A		(PINC & 1<<PC0)
#define PHASE_B		(PINC & 1<<PC1)
 

volatile int8_t enc_delta;			// -128 ... 127
static int8_t last;

void encode_init( void )
{
  int8_t new;
 
  new = 0;
  if( PHASE_A )
    new = 3;
  if( PHASE_B )
    new ^= 1;					// convert gray to binary
  last = new;					// power on state
  enc_delta = 0;
  TCCR0 = 1<<WGM01^1<<CS01^1<<CS00;		// CTC, XTAL / 64
  OCR0 = (uint8_t)(XTAL / 64.0 * 1e-3 - 0.5);	// 1ms
  TIMSK |= 1<<OCIE0;
}
 
 
ISR( TIMER0_COMP_vect )				// 1ms for manual movement
{
  int8_t new, diff;
 
  new = 0;
  if( PHASE_A )
    new = 3;
  if( PHASE_B )
    new ^= 1;					// convert gray to binary
  diff = last - new;				// difference last - new
  if( diff & 1 ){				// bit 0 = value (1)
    last = new;					// store new as next last
    enc_delta += (diff & 2) - 1;		// bit 1 = direction (+/-)
  }
}
 
 
int8_t encode_read1( void )			// read single step encoders
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = 0;
  sei();
  return val;					// counts since last call
}
 
 
int8_t encode_read2( void )			// read two step encoders
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = val & 1;
  sei();
  return val >> 1;
}
 
 
int8_t encode_read4( void )			// read four step encoders
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = val & 3;
  sei();
  return val >> 2;
}
 
 
int main(void)
{
	int32_t val = 0;
	int32_t val_old=0;
  
    char buffer[7];
    /*
     *  Initialize UART library, pass baudrate and AVR cpu clock
     *  with the macro 
     *  UART_BAUD_SELECT() (normal speed mode )
     *  or 
     *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
     */
  
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
    
        
	encode_init();    
    
	sei();
	
	for(;;){
		val += encode_read2();			// read a single step encoder

		if(val != val_old){
			itoa( val, buffer, 10);   // convert interger into string (decimal format)         
			uart_puts(buffer);        // and transmit string to UART
			uart_putc(10);
			uart_putc(13);
			val_old=val;
		}
	}
    
}
